@@ -3,7 +3,7 @@
 
 from usecase.guishared import GuiEvent, MethodIntercept
 from usecase.definitions import UseCaseScriptError
-import gtk
+import gtk, gobject
 
 # Abstract Base class for all GTK events
 class GtkEvent(GuiEvent):
@@ -11,8 +11,15 @@ class GtkEvent(GuiEvent):
         GuiEvent.__init__(self, name, widget)
         self.interceptMethod(self.widget.stop_emission, EmissionStopIntercept)
         self.interceptMethod(self.widget.emit_stop_by_name, EmissionStopIntercept)
+        self.interceptMethod(self.widget.get_toplevel().destroy, DestroyIntercept)
         self.stopEmissionMethod = None
-        
+        self.destroyMethod = None
+
+    @staticmethod
+    def disableIntercepts(window):
+        if isinstance(window.destroy, MethodIntercept):
+            window.destroy = window.destroy.method
+
     @classmethod
     def getAssociatedSignal(cls, widget):
         return cls.signalName
@@ -26,20 +33,9 @@ class GtkEvent(GuiEvent):
         if widget.isInstanceOf(gtk.TreeView):
             # Ignore this for treeviews: as they have no title/label they can't really get confused with other stuff
             return widget.get_model() is not None
-
-        # We tried using gobject.type_name and gobject.signal_list_names but couldn't make it work
-        # We go for the brute force approach : actually do it and remove it again and see if we succeed...
-        try:
-            def nullFunc(*args) : pass
-            if hasattr(widget, "connect_for_real"): # convention for when we intercept connect, as with dialogs
-                handler = widget.connect_for_real(signalName, nullFunc)
-            else:
-                handler = widget.connect(signalName, nullFunc)
-            widget.disconnect(handler)
-            return True
-        except TypeError:
-            return False
-
+        else:
+            return gobject.signal_lookup(signalName, widget.widget) != 0
+        
     def delayLevel(self):
         # If we get this when in dialog.run, the event that cause us has not yet been
         # recorded, so we should delay
@@ -54,16 +50,18 @@ class GtkEvent(GuiEvent):
 
     def _connectRecord(self, gobj, method):
         handler = gobj.connect(self.getRecordSignal(), method, self)
-        gobj.connect(self.getRecordSignal(), self.stopEmissions)
+        gobj.connect(self.getRecordSignal(), self.executePostponedActions)
         return handler
 
     def outputForScript(self, widget, *args):
         return self._outputForScript(*args)
 
-    def stopEmissions(self, *args):
+    def executePostponedActions(self, *args):
         if self.stopEmissionMethod:
             self.stopEmissionMethod(self.getRecordSignal())
             self.stopEmissionMethod = None
+        if self.destroyMethod:
+            self.destroyMethod()
 
     def shouldRecord(self, *args):
         return GuiEvent.shouldRecord(self, *args) and self.widget.get_property("visible")
@@ -97,6 +95,15 @@ class EmissionStopIntercept(MethodIntercept):
             if stdSigName == event.getRecordSignal():
                 event.stopEmissionMethod = self.method
 
+class DestroyIntercept(MethodIntercept):
+    inResponseHandler = False    
+    def __call__(self):
+        if self.inResponseHandler:
+            self.method()
+        else:
+            for event in self.events:
+                event.destroyMethod = self.method
+
         
 # Generic class for all GTK events due to widget signals. Many won't be able to use this, however
 class SignalEvent(GtkEvent):
