@@ -91,11 +91,15 @@ class UseCaseReplayer:
     
     def addScript(self, script):
         self.scripts.append(script)
+        if self.processInitialWait(script):
+            self.enableReading()
+
+    def processInitialWait(self, script):
         waitCommand = script.getCommand(waitCommandName)
         if waitCommand:
-            self.processWait(self.getArgument(waitCommand, waitCommandName))
+            return self.processWait(self.getArgument(waitCommand, waitCommandName))
         else:
-            self.enableReading()
+            return True
         
     def enableReading(self):
         # By default, we create a separate thread for background execution
@@ -216,10 +220,13 @@ class UseCaseReplayer:
     def parseCommand(self, scriptCommand):
         commandName = self.findCommandName(scriptCommand)
         if not commandName:
-            raise UseCaseScriptError, "Could not parse script command '" + scriptCommand + "'"
+            raise UseCaseScriptError, self.getParseError(scriptCommand)
         argumentString = self.getArgument(scriptCommand, commandName)
         return commandName, argumentString
 
+    def getParseError(self, scriptCommand):
+        return "Could not parse script command '" + scriptCommand + "'"
+
     def getArgument(self, scriptCommand, commandName):
         return scriptCommand.replace(commandName, "").strip()
 
